'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var axios = require('axios');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

/**
 * This class handles the http requests using axios
 * @version 1.0.0
 * @author Alejandro <alejandro.devop@gmail.com>
 */
var HttpClient = /** @class */ (function () {
    function HttpClient(options) {
        /** This are the app headers, can be modified using the setter function */
        this.headers = {
            Accept: 'application/json',
            'Content-Type': 'application/json'
        };
        /** Here's where the client stores the requests send */
        this.requestsLog = [];
        /** Flag to control if the application should be logged */
        this.log = false;
        /** The endpoints or paths to make the requests */
        this.endpoints = {};
        var _a = options || {}, auth = _a.auth, server = _a.server, endpoints = _a.endpoints, log = _a.log, onRequestDone = _a.onRequestDone;
        this.server = server;
        this.log = log;
        this.endpoints = endpoints || {};
        this.onRequestDone = onRequestDone;
        if (auth) {
            this.headers['Authorization'] = auth;
        }
    }
    /**
     * Function to get the client headers
     */
    HttpClient.prototype.getHeaders = function () {
        return this.headers;
    };
    /**
     * Function to get the client endpoints
     */
    HttpClient.prototype.getEndpoints = function () {
        return this.endpoints;
    };
    /**
     * Function to get the client server url
     */
    HttpClient.prototype.getServer = function () {
        return this.server;
    };
    /**
     * Function to update the client properties
     * auth: Adds a header to control the authorization
     * @param properties
     */
    HttpClient.prototype.setProperties = function (properties) {
        var auth = (properties || {}).auth;
        this.headers['Authorization'] = auth;
    };
    /**
     * Function to extract paths from the endpoints map
     * @param path
     * @param options
     * @param method
     */
    HttpClient.prototype.resolvePath = function (path, options, method) {
        var _a = options || {}, replace = _a.replace, _b = _a.replaceToken, replaceToken = _b === void 0 ? ':' : _b;
        if (!path.includes('.')) {
            return path;
        }
        var parts = path.split('.');
        var url = parts
            .reduce(function (newValue, currentPart) {
            if (newValue[currentPart]) {
                newValue = newValue[currentPart];
            }
            return newValue;
        }, this.endpoints)
            .toString();
        if (replace) {
            Object.keys(replace).forEach(function (currentKey) {
                url = url.replace(new RegExp("" + replaceToken + currentKey + replaceToken), replace[currentKey]);
            });
        }
        /**
         * If the url has type definition we must clean it
         */
        if (url.includes('<') && url.includes('>')) {
            url = url.replace(new RegExp("<.*>", 'g'), '');
        }
        /**
         * Check if the route has method validation, you can add method validation for paths by prepending
         * [method]:/my/route to your routes, this ensures that no one calls a post method using a get hook
         */
        if (url.includes(':')) {
            var _c = url.split(':'), methodToValidate = _c[0], pathRest = _c[1];
            if (method !== methodToValidate) {
                throw new Error("Invalid method " + method + " for path of type " + methodToValidate);
            }
            else {
                url = pathRest;
            }
        }
        return url.toString();
    };
    /**
     * Function to execute any kind of request
     * @param path
     * @param options
     */
    HttpClient.prototype.doRequest = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, method, params, debug, data, auth, url, response, requestHeaders, requestConfig, e_1;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _a = options || {}, _b = _a.method, method = _b === void 0 ? 'get' : _b, params = _a.params, debug = _a.debug, data = _a.data, auth = _a.auth;
                        url = "" + this.server + this.resolvePath(path, options, method);
                        response = null;
                        _c.label = 1;
                    case 1:
                        _c.trys.push([1, 12, , 13]);
                        requestHeaders = __assign(__assign({}, this.headers), (auth ? { Authorization: auth } : {}));
                        requestConfig = {
                            headers: requestHeaders,
                            responseType: 'json'
                        };
                        if (!(method === 'get')) return [3 /*break*/, 3];
                        requestConfig.params = params;
                        return [4 /*yield*/, axios__default['default'].get(url, requestConfig)];
                    case 2:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 3:
                        if (!(method === 'post')) return [3 /*break*/, 5];
                        return [4 /*yield*/, axios__default['default'].post(url, data, requestConfig)];
                    case 4:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 5:
                        if (!(method === 'patch')) return [3 /*break*/, 7];
                        return [4 /*yield*/, axios__default['default'].patch(url, data, requestConfig)];
                    case 6:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 7:
                        if (!(method === 'delete')) return [3 /*break*/, 9];
                        return [4 /*yield*/, axios__default['default'].delete(url, requestConfig)];
                    case 8:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 9:
                        if (!(method === 'put')) return [3 /*break*/, 11];
                        return [4 /*yield*/, axios__default['default'].put(url, data, requestConfig)];
                    case 10:
                        response = _c.sent();
                        _c.label = 11;
                    case 11:
                        if (this.log) {
                            this.logRequest({
                                response: response,
                                headers: requestHeaders,
                                url: url,
                                method: method
                            });
                        }
                        if (this.onRequestDone) {
                            this.onRequestDone({
                                response: response,
                                loggedRequests: this.getLog()
                            });
                        }
                        return [2 /*return*/, debug
                                ? {
                                    method: method,
                                    headers: this.headers,
                                    body: data,
                                    data: response,
                                    urlParams: params,
                                    url: url
                                }
                                : response];
                    case 12:
                        e_1 = _c.sent();
                        throw e_1;
                    case 13: return [2 /*return*/];
                }
            });
        });
    };
    HttpClient.prototype.logRequest = function (requestInfo) {
        var method = requestInfo.method, headers = requestInfo.headers, url = requestInfo.url, response = requestInfo.response;
        this.requestsLog = __spreadArray([
            {
                method: method,
                headers: headers,
                url: url,
                response: response
            }
        ], this.requestsLog);
    };
    /**
     * Function to execute a GET request
     * @param path
     * @param options
     */
    HttpClient.prototype.doGet = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'get' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_2 = _a.sent();
                        throw e_2;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a POST request
     * @param path
     * @param options
     */
    HttpClient.prototype.doPost = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'post' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_3 = _a.sent();
                        throw e_3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a PATCH request
     * @param path
     * @param options
     */
    HttpClient.prototype.doPatch = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_4;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'patch' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_4 = _a.sent();
                        throw e_4;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a PUT
     * @param path
     * @param options
     */
    HttpClient.prototype.doPut = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_5;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'put' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_5 = _a.sent();
                        throw e_5;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a Delete Request
     * @param path
     * @param options
     */
    HttpClient.prototype.doDelete = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_6;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'delete' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_6 = _a.sent();
                        throw e_6;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to retrieve the application requests log
     */
    HttpClient.prototype.getLog = function () {
        return this.requestsLog;
    };
    HttpClient.prototype.setEndpoints = function (endpoints) {
        this.endpoints = endpoints;
    };
    return HttpClient;
}());

var ApiContext = React__default['default'].createContext({});
var ApiProvider = ApiContext.Provider;
var useState = React__default['default'].useState;
/**
 * This component provides the HTTP client instance to all application
 * @author Alejandro <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param children
 * @param config
 * @param ClientLogger
 * @constructor
 */
var ApiClient = function (_a) {
    var children = _a.children, config = _a.config, dataProcessor = _a.dataProcessor, ClientLogger = _a.ClientLogger;
    var _b = config || {}, auth = _b.auth, debug = _b.debug, endpoints = _b.endpoints, server = _b.server, log = _b.log, strictJSON = _b.strictJSON;
    var _c = useState([]), requestsLog = _c[0], setRequestsLog = _c[1];
    var client = new HttpClient({
        dataProcessor: dataProcessor,
        strictJSON: strictJSON,
        server: server,
        endpoints: endpoints,
        debug: debug,
        log: log,
        auth: auth,
        onRequestDone: function (_a) {
            var loggedRequests = _a.loggedRequests;
            if (log) {
                setRequestsLog(loggedRequests);
            }
        }
    });
    var setClientProperties = function (properties) {
        client.setProperties(properties);
    };
    return (React__default['default'].createElement(ApiProvider, { value: { client: client, dataProcessor: dataProcessor, setClientProperties: setClientProperties, strictJSON: strictJSON } },
        children,
        log && ClientLogger && React__default['default'].createElement(ClientLogger, { requests: requestsLog })));
};

/**
 * Hook to allow access Api client and api context values
 * @author Alejandro <alejandro.devop@gmail.com>
 * @version 1.0.0
 */
var useApiSettings = function () {
    var _a = React.useContext(ApiContext), requestsLog = _a.requestsLog, setClientProperties = _a.setClientProperties;
    return { setClientProperties: setClientProperties, requestsLog: requestsLog };
};

/**
 * Hook to execute get Requests, if you want to execute it lazy passed the flag lazy to true
 * if lazy it will return an array, first position the getter function which will be a promise, the second
 * position it's the loading state
 * otherwise the hook will return an object, data position it's the response, and the loading position represents
 * the loading state, the request will be triggered on Mount
 * @author Alejandro <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param path
 * @param config
 * @param lazy
 */
var useGet = function (path, config) {
    // The loading flag and the response data are stored in the same state to avoid create multiple setStates and Multiple SetState calls
    var _b = React.useState({
        loading: true
    }), _c = _b[0], data = _c === void 0 ? {} : _c, setData = _b[1];
    var _d = React.useContext(ApiContext), client = _d.client, dataProcessor = _d.dataProcessor;
    var _e = data, loading = _e.loading, responseData = _e.data;
    var sendRequest = React.useCallback(function (configOverride) { return __awaiter(void 0, void 0, void 0, function () {
        var response, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!loading) {
                        setData(__assign({ loading: true }, data));
                    }
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, client.doGet(path, __assign(__assign({}, config), configOverride))];
                case 2:
                    response = _a.sent();
                    if (dataProcessor && dataProcessor instanceof Function) {
                        response = dataProcessor(response);
                    }
                    setData({ loading: false, data: response.data, total: response.total || 0 });
                    return [2 /*return*/, response.data];
                case 3:
                    e_1 = _a.sent();
                    setData(__assign({ loading: false }, data));
                    throw e_1;
                case 4: return [2 /*return*/];
            }
        });
    }); }, [config, path, setData]);
    React.useEffect(function () {
        sendRequest();
    }, []);
    return React.useMemo(function () { return ({ data: responseData, loading: loading, refetch: sendRequest }); }, [loading, responseData, sendRequest]);
};

/**
 *
 * @author Alejandro <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param path
 * @param config
 */
var useGetLazy = function (path, config) {
    // The loading flag and the response data are stored in the same state to avoid create multiple setStates and Multiple SetState calls
    var _a = React.useState({
        loading: false
    }), _b = _a[0], data = _b === void 0 ? {} : _b, setData = _a[1];
    var _c = React.useContext(ApiContext), client = _c.client, dataProcessor = _c.dataProcessor;
    var loading = data.loading;
    var sendRequest = React.useCallback(function (configOverride) { return __awaiter(void 0, void 0, void 0, function () {
        var response, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!loading) {
                        setData({ loading: true });
                    }
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, client.doGet(path, __assign(__assign({}, config), configOverride))];
                case 2:
                    response = _a.sent();
                    if (dataProcessor && dataProcessor instanceof Function) {
                        response = dataProcessor(response);
                    }
                    setData({ loading: false });
                    return [2 /*return*/, response];
                case 3:
                    e_1 = _a.sent();
                    setData({ loading: false });
                    throw e_1;
                case 4: return [2 /*return*/];
            }
        });
    }); }, [config, path, setData]);
    return [sendRequest, loading];
};

/**
 * Hook to execute a post request
 * @param path
 * @param config
 */
var usePost = function (path, config) {
    var _a = React.useState(false), loading = _a[0], setLoading = _a[1];
    var _b = React.useContext(ApiContext), client = _b.client, strictJSON = _b.strictJSON;
    var sendRequest = React.useCallback(function (_a) {
        var payload = _a.payload, id = _a.id, configOverride = _a.config;
        return __awaiter(void 0, void 0, void 0, function () {
            var _b, entityType, formData, device, data, bodyFormData_1, method, response, e_1;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _b = config || {}, entityType = _b.entityType, formData = _b.formData, device = _b.device;
                        data = {};
                        if (formData && payload) {
                            bodyFormData_1 = new FormData();
                            Object.keys(payload).forEach(function (fieldKey) {
                                bodyFormData_1.append(fieldKey, payload[fieldKey]);
                            });
                            bodyFormData_1.append('device_name', device || '');
                            data = bodyFormData_1;
                        }
                        else if (strictJSON) {
                            data = {
                                data: {
                                    type: entityType,
                                    id: id,
                                    attributes: payload
                                }
                            };
                        }
                        else {
                            data = payload;
                        }
                        setLoading(true);
                        _c.label = 1;
                    case 1:
                        _c.trys.push([1, 10, , 11]);
                        method = (config || {}).method;
                        response = null;
                        if (!(method === 'put')) return [3 /*break*/, 3];
                        return [4 /*yield*/, client.doPut(path, __assign(__assign(__assign({}, config), { data: data }), configOverride))];
                    case 2:
                        response = _c.sent();
                        return [3 /*break*/, 9];
                    case 3:
                        if (!(method === 'patch')) return [3 /*break*/, 5];
                        return [4 /*yield*/, client.doPatch(path, __assign(__assign(__assign({}, config), { data: data }), configOverride))];
                    case 4:
                        response = _c.sent();
                        return [3 /*break*/, 9];
                    case 5:
                        if (!(method === 'delete')) return [3 /*break*/, 7];
                        return [4 /*yield*/, client.doDelete(path, __assign(__assign(__assign({}, config), { data: data }), configOverride))];
                    case 6:
                        response = _c.sent();
                        return [3 /*break*/, 9];
                    case 7: return [4 /*yield*/, client.doPost(path, __assign(__assign(__assign({}, config), { data: data }), configOverride))];
                    case 8:
                        response = _c.sent();
                        _c.label = 9;
                    case 9:
                        setLoading(false);
                        return [2 /*return*/, response];
                    case 10:
                        e_1 = _c.sent();
                        setLoading(false);
                        throw e_1;
                    case 11: return [2 /*return*/];
                }
            });
        });
    }, [setLoading, config, path]);
    return [sendRequest, loading];
};

/**
 * Hook to execute a patch request, it uses as base the usePost hook
 * @author Alejandro <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param path
 * @param config
 */
var usePatch = function (path, config) {
    return usePost(path, __assign(__assign({}, (config || {})), { method: 'patch' }));
};

/**
 * Hook to execute a put request, it uses as base the usePost hook
 * @param path
 * @param config
 */
var usePut = function (path, config) {
    return usePost(path, __assign(__assign({}, (config || {})), { method: 'put' }));
};

/**
 * Hook to allow execute delete requests, it uses as base the usePost hook
 * @author Alejandro <alejandro.devop@gmail.com>
 * @param path
 * @param config
 */
var useDelete = function (path, config) {
    return usePost(path, __assign(__assign({}, (config || {})), { method: 'delete' }));
};

exports.default = ApiClient;
exports.useApiSettings = useApiSettings;
exports.useDelete = useDelete;
exports.useGet = useGet;
exports.useGetLazy = useGetLazy;
exports.usePatch = usePatch;
exports.usePost = usePost;
exports.usePut = usePut;
//# sourceMappingURL=index.js.map
